#include <stdio.h>
#include <getopt.h>
#include <stdlib.h>
#include <string.h>

struct partition_entry {
	unsigned char boot;

	unsigned char start_head;
	unsigned short start_cyl_sec;

	unsigned char part_type;

	unsigned char end_head;
	unsigned short end_cyl_sec;

	unsigned int relative_sectors;
	unsigned int total_sectors;
};

void print_usage(void)
{
        fprintf(stderr, "Usage: genpart [Options]\n\n"
			"Generate a 16 byte partition table entry defined by command line\n"
			"arguments and dump it to stdout. No CHS magic is done, only lba\n"
			"entries are filled out.\n"
			"\n"
                        "Options:\n"
			" -b, --begin=<block>    start block of this partition\n"
			" -s, --size=<blocks>    size of this partition\n"
			" -t  --type=<type       partition type\n"
			" -c, --bootable=<0|1>   bootable flag\n"
			" -m, --magic            append a 0x55aa magic\n"
			" -h, --help             this help\n"
	);
}

int main(int argc, char *argv[])
{
	struct partition_entry entry;
	int opt, do_magic = 0;
	unsigned short magic = 0xaa55;

	struct option long_options[] = {
		{ "help", no_argument, 0, 'h' },
		{ "begin", required_argument, 0, 'b'},
		{ "size", required_argument, 0, 's'},
		{ "bootable", no_argument, 0, 'c'},
		{ "type", required_argument, 0, 't'},
		{ "magic", no_argument, 0, 'm'},
		{ 0, 0, 0, 0},
	};

	if(argc == 1) {
		print_usage();
		exit(0);
	}

	memset(&entry, 0, sizeof(struct partition_entry));

	entry.part_type = 0x83; /* default to linux */
	entry.start_head = 0xff;
	entry.start_cyl_sec = 0xffff;
	entry.end_head = 0xff;
	entry.end_cyl_sec = 0xffff;

	while ((opt = getopt_long(argc, argv, "hb:s:ct:m", long_options, NULL)) != -1) {
		switch (opt) {
			case 'h':
				print_usage();
				exit(0);
			case 'b':
				entry.relative_sectors = strtoul(optarg, NULL, 0);
				break;
			case 's':
				entry.total_sectors = strtoul(optarg, NULL, 0);
				break;
			case 'c':
				entry.boot = 0x80;
				break;
			case 't':
				entry.part_type = strtoul(optarg, NULL, 0);
			case 'm':
				do_magic = 1;
				break;
		}
	}

	fwrite(&entry, sizeof(struct partition_entry), 1, stdout);

	if(do_magic)
		fwrite(&magic, 2, 1, stdout);

	return 0;
}
